# %W% %G% %U% - %Q% #
#ident "@(#)VCSOR:%M% %I%"
#
# $Copyright: Copyright (c) 2010 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
 
package oraclelib;
use strict;

use lib qw(.);
use oraconfig () ;
use vars qw(%CONF);
*CONF =\%oraconfig::CONF;
my $CAT = $CONF{CAT};
my $SU = $CONF{SU};
my $PS = $CONF{PS};
my $GREP = $CONF{GREP};
my $LIBRARY_PATH = $CONF{LIBRARY_PATH};
my $AGENT_DIR = "/opt/VRTSagents/ha";
my $MYVCSHOME = "";
my $CLUS_PROD = "";

$CLUS_PROD = $ENV{"CLUSTER_PRODUCT"};
if (! defined($CLUS_PROD))
{
    $CLUS_PROD = "VCS";
}

if ($CLUS_PROD eq "VAD")
{
    $MYVCSHOME = $ENV{"NGC_HOME"} || "/opt/VRTSvad";
}
else
{
    $MYVCSHOME = $ENV{"VCS_HOME"} || "/opt/VRTSvcs";
}
	
################# FUNCTION DEFN START ##########################
# Function : check_oracle_home
# Desc     : Checks for inexistence of ORACLE_HOME 
# Input    : $Home 
# Output   : None
# Returns  : (1) -  home does not exist  
#
sub check_oracle_home  {

	my ($Home) =@_;
	unless ( -d $Home ) {
		return  1;
	}
	return 0;
}

# Function : check_sqlplus
# Desc     : Checks if sqlplus does not exist .
# Desc     : If they exist , this function sets DBAMGR and CONNECTSTR
# Input    : $Home
# Output   : $DBAMGR,$CONNECTSTR - changes values of these variables
# Returns  : (1) - no suitable client exists
#
sub check_sqlplus {
my ($Home,$DBAMGR,$CONNECTSTR, $version, $release)=@_;
my $agent_type = $ENV{"VCS_LOG_AGENT_NAME"};
my $ret;

        if ( -x "$Home/bin/sqlplus" ) {
                $DBAMGR = "$Home/bin/sqlplus -s /nolog";
		if ( $agent_type eq "Oracle") {
                	$CONNECTSTR = "connect / as sysdba";
		} elsif ( $agent_type =~ /ASM/ ) {
			$ret= find_version($Home, $version, $release);
			if ( $version >= 11 ) {
				# For 11g and above, use sysasm
				$CONNECTSTR = "connect / as sysasm";
			} else {
				# For 10g and below, use sysdba
				$CONNECTSTR = "connect / as sysdba";
			}
		}
                $_[1] = $DBAMGR;
                $_[2] = $CONNECTSTR;
		$_[3] = $version;
		$_[4] = $release;
        } else {
                return 1 ;
        }
        return 0;
}

# Function : check_oracle_client
# Desc     : Checks if svrmgrl/sqlplus does not exist .
# Desc     : If they exist , this function sets DBAMGR and CONNECTSTR
# Input    : $Home  
# Output   : $DBAMGR,$CONNECTSTR - changes values of these variables 
# Returns  : (1) - no suitable client exists 
#
sub check_oracle_client {
my ($Home,$DBAMGR,$CONNECTSTR)=@_;

	if ( -x "$Home/bin/svrmgrl" ) {
		$DBAMGR = "$Home/bin/svrmgrl";
		$CONNECTSTR = "connect internal";
		$_[1] = $DBAMGR;
		$_[2] = $CONNECTSTR;
	} elsif ( -x "$Home/bin/sqlplus" ) {
		#
		# 'svrmgrl' removed and 'connect internal' usage discontinued in 9i.
		# Hence, to have Operating System authentication, use '/ as sysdba'
		# i.e. connect as a privileged default user. '/' represents a default
		# logon using OS authentication.
		#
		$DBAMGR = "$Home/bin/sqlplus -s /nolog";
		$CONNECTSTR = "connect / as sysdba";
		$_[1] = $DBAMGR;
		$_[2] = $CONNECTSTR;
	} else {
		return 1 ;
	}
	return 0;
}

# Function : check_srvctl
# Desc     : Checks if srvctl does not exist (RAC installations) .
# Desc     : If they exist , this function sets DBAMGR 
# Input    : $Home  
# Output   : $DBAMGR - changes values of these variables 
# Returns  : (1) - no suitable client exists 
#
sub check_srvctl {
my ($Home,$DBAMGR)=@_;

	if ( -x "$Home/bin/srvctl" ) {
		$DBAMGR = "$Home/bin/srvctl";
		$_[1] = $DBAMGR;
	}   else {
		return 1 ;
	}
	return 0;
}


# Function : set_environment
# Desc     : Checks the EnvFile, if not valide sets $ValidFile to 1
# Desc     : Else sets the Oracle related env vars 
# Input    : $UseCSH - shell of the owner ( 1 if csh)  
# Input    : $EnvFile - source file for env vars
# Input    : $Owner 
# Input    : $Home 
# Output   : $ValidFile - 
# Returns  :  $LD_ENV - Env vars in format to be set 
# Note     :  This function sets LD_LIBRARY_PATH ( and equivalents) 
#          :  TWO_TASK, extra env vars 
#
sub set_environment {
	my ($UseCSH, $EnvFile, $Owner, $Home, $SID, $ValidFile) =@_;
	my ($LD_ENV);
	my $EnvFileExists = 0;
	my $caterr = "";

	if ( $EnvFile ) {
		
		$_[5] = 0 ;	#is a valid file
		#
		# It is the user's responsibility to ensure that EnvFile can
		# be sourced successfully using the login shell of $User. 
		#
		# First ensure that the file is a Plain file AND a Text file.
		# Other file types (e.g. Named pipe) etc. cause $EnvFile to be
		# NOT sourced, with a message in the engine log saying so.
		#
		if ( -f $EnvFile && -T $EnvFile ) {
			#
			# Doing a 'cat' on the file as user '$Owner' ensures that the 
			# permission are correct for user '$Owner'.
			# Execute permissions are NOT needed for $EnvFile to source it.
			#
			# Notes:
			#   * 2>&1 is for the su and NOT for cat.
			#   * `` always invokes sh.
			#   * > /dev/null is for the cat command.
			#   * The cat command syntax used is shell-neutral.
			#
			$caterr = `$SU $Owner -c "$CAT $EnvFile > /dev/null" 2>&1`;
			if ( $caterr ) {
				$EnvFileExists = 0;	# error encountered by cat
				$_[5]=1; 		# Not a valid file
			} else {
				$EnvFileExists = 1;	# cat successful
			}
		} else {
			$EnvFileExists = 0;		# not a text file
			$_[5]=1; 			# Not a valid file
		}
	}
	# Important points from the man page of 'su':
	#   * Regarding LD_LIBRARY_PATH,
	#       Variables with LD_ prefix are removed for security  reasons.
	#       Thus,  su  bin will not retain previously exported variables
	#       with LD_ prefix while becoming user bin.
	#   * The environment  of  user root is passed along to user oracle,
	#       with the exception of $PATH, which is controlled by PATH and
	#       SUPATH in /etc/default/su.
	#   * $LANG environment variable: The su should work even if LANG=ja:
	#       If any  of  the  LC_*  variables  (  LC_CTYPE,  LC_MESSAGES,
	#       LC_TIME,   LC_COLLATE,  LC_NUMERIC,  and  LC_MONETARY)  (see
	#       environ(5)) are not set in the environment, the  operational
	#       behavior  of  su  for  each corresponding locale category is
	#       determined by the value of the LANG environment variable. If
	#       LC_ALL  is  set,  its contents are used to override both the
	#       LANG and the other LC_* variables.  If  none  of  the  above
	#       variables  are  set in the environment, the "C" (U.S. style)
	#       locale determines how su behaves.

	# NOTE: that the LD_LIBRARY_PATH should be set AFTER the su to $Owner, and
	#	NOT for environment of the user (root) running this script.
	# Be careful with the backslashes in this script.
	#
	my ($ldlibpath);
	if ( -d "$Home/lib64" ) {
		 $ldlibpath="/usr/lib:$Home/lib64:\\\$$LIBRARY_PATH";
	} else {
		$ldlibpath="/usr/lib:$Home/lib:\\\$$LIBRARY_PATH";
	}
	
	# If CSH  then setenv else export environment variables 
	if ($UseCSH) {
		$LD_ENV = "setenv $LIBRARY_PATH \\\"\\\"; ";
		#
		# Note the .= concatenation operator below
		#
		if ($EnvFileExists) {
			$LD_ENV .= "source $EnvFile; ";
		}
		$LD_ENV .= "setenv $LIBRARY_PATH $ldlibpath;unsetenv TWO_TASK; setenv ORACLE_HOME $Home; setenv ORACLE_SID $SID";
	} else {
		$LD_ENV = "$LIBRARY_PATH=\"\"; export $LIBRARY_PATH; ";
		#
		# Note the .= concatenation operator below
		#
		if ($EnvFileExists) {
			$LD_ENV .= ". $EnvFile; ";
		}
		$LD_ENV .= "$LIBRARY_PATH=$ldlibpath; export $LIBRARY_PATH ;TWO_TASK=; export TWO_TASK; ORACLE_HOME=$Home; export ORACLE_HOME; ORACLE_SID=$SID; export ORACLE_SID";
	}
	return $LD_ENV;
}

# Function : runoraapi
# Desc     : Execute oraapi_bin 
# Input    : $Owner - owner of executables
#          : $LD_ENV - Env var string returned from function "check_environment"
# Output   : Exit code 
# Returns  : Exit code
#
sub runoraapi {
        my($Owner, $LD_ENV, $agent_type, $type, $version, $use_csh) = @_;
        my $OH = $ENV{"ORACLE_HOME"};
        my $OS = $ENV{"ORACLE_SID"};
        my $out = "";
        my $exit_status = 98;
	my $oraapi_bin_output = "";
        my $vcs_home=$AGENT_DIR;
	my $bits=32;
	my $oracle_binary = "$OH/bin/oracle";
	my $exec_name = "";
        $vcs_home = "/opt/VRTSvcs" if (!$vcs_home);

	if ( $version eq "10g" ) {
		$version="";
	}
	if ( -x $oracle_binary ) {
		$bits = `file $oracle_binary`;
		chop($bits);
		$bits =~ s/.*(32|64)-bit.*/$1/;
	}
        if ( ($bits !~ m/^\d+$/) or (($bits != 32) and ($bits != 64)) ) {
                $bits = 64;   
        }

       	$exec_name = $type . "api_$bits". $version;  
	if (! -e "$vcs_home/bin/$agent_type/$exec_name" )  {
		$exit_status = 97;
		$oraapi_bin_output = $bits;
	} else {
		if ( $use_csh eq 0 ) {
			$out = `$SU $Owner -c "$LD_ENV; $vcs_home/bin/$agent_type/$exec_name; echo \\\$?"`;
		} else {
			$out = `$SU $Owner -c "$LD_ENV; $vcs_home/bin/$agent_type/$exec_name; echo \\\$status"`;
		}
		$out =~ m/(.*)\n(\d+)\n$/;
		$oraapi_bin_output = "$1" ;
		$exit_status = $2;
	}

        return ($exit_status, $oraapi_bin_output);

} # End of sub runoraapi()

# Function : check_oracle_query_client
# Desc     : sets the client to query /update the database ( put correct incident number here)
# Desc     : If they exist , this function sets DBAMGR and CONNECTSTR
# Input    : $Home
# Output   : $DBAMGR,$CONNECTSTR - changes values of these variables
# Returns  : (1) - no suitable client exists
#
sub check_oracle_query_client {
	my ($Home, $DBAMGR, $CONNECTSTR, $LD_ENV)=@_;

	my $ret=0;
	if ( -x "$Home/bin/svrmgrl") {
		my ($out, @out1);
		$out=`"$LD_ENV; $Home/bin/svrmgrl"<<EOF
exit;
EOF`;
		@out1=split(/\n/,$out); 
		my (@substr ,@value , $version, $string, $lineval);
		foreach $lineval (@out1) {
			chomp $lineval;
			if($lineval=~/Oracle/ && $lineval=~/Release/ && $lineval =~ /Edition/) {
				$string = $lineval;
				@substr=split(/ /,$string);
				foreach my $str(@substr) {
					if ($str =~/^\d/) {
						$version=$str;
						last;
					}
				}
			last; 
			} # end of if
		} # end of foreach

		# ret Value. 
		# 0  -- > 8i and less , then call svrmgrl (8.0.x.x),else
		# 1  -- > greater than  or equal 8i, call sqlplus (8.1.x.x)

		@value=split(/\./,$version);

		if($value[0] < 8) {
			$ret=0;
		} elsif($value[0] eq 8 && $value[1] < 1) {
			$ret=0;
		} else {
			$ret=1;
		}
	} # end of mainif
	else  { # when svrmgrl file not present.
		$ret=1;
	}
	if($ret eq 1) {
		if ( -x "$Home/bin/sqlplus" ) {
			#
			# 'svrmgrl' removed and 'connect internal' usage discontinued in 9i.
			# Hence, to have Operating System authentication, use '/ as sysdba'
			# i.e. connect as a privileged default user. '/' represents a default
			# logon using OS authentication.
			#
			$DBAMGR = "$Home/bin/sqlplus -s /nolog";
			$CONNECTSTR = "connect / as sysdba";
			$_[1] = $DBAMGR;
			$_[2] = $CONNECTSTR;
		}
		else {
			$ret=0; # try svrmgrl
		}
	}
	elsif ( $ret eq 0 ) {
		if ( -x "$Home/bin/svrmgrl" ) {
			$DBAMGR = "$Home/bin/svrmgrl";
			$CONNECTSTR = "connect internal";
			$_[1] = $DBAMGR;
			$_[2] = $CONNECTSTR;
		}
		else{
			return 1; # even svrmgrl fails
		} # end of elsif
	}
	return 0;
}

# Function : cleanconnections
# Desc     : Remove Oracle connection processes which may have been left around by an aborted instance
# Input    : $SID - Instance of executables
# Returns  : (0) - Did not clean becoz: (a) could not open pipe
#          :  else return process ids
#
sub cleanconnections {
	my ($SID) =@_;
	my ($out,$pid);
	my @clientpids = ();

	# Get current Oracle Process IDs
	#
	unless (open(PS, "$PS |")) {
		return 0;
	}

	while (<PS>) {
		next unless /\boracle$SID\b/;
		($pid) = split;
		push @clientpids, $pid;
	}
	close(PS);
	$out="";
	if ( @clientpids ) {
		if ( kill(9, @clientpids) > 0 ) {
			$out = join(", ", @clientpids);
		}
		return $out;
	}
	return 0;
}


# Function : cleanipc
# Desc     : Remove IPC resources which may have been left around by an aborted instance  
# Input    : $Owner - owner of executables
#          : $SID - instance
#          : $Home - ORACLE_HOME
#          : $LD_ENV - Environment variables set for executing clients
# Output   : None 
# Returns  : (0) - Did not clean becoz: (a) Home does not exist 
#          :                            (b) sysresv does not exist 
#          :  else return output
#
sub cleanipc {
my ($Owner,$SID,$Home,$LD_ENV) =@_;
my ($out);
	if ( ! -d $Home ) {
		return 0;
	}
	my $SYSRESV = "$Home/bin/sysresv";
	if ( -x $SYSRESV ) {
		$out = `$SU $Owner -c "$LD_ENV; $SYSRESV -f -d on -l $SID" 2>&1`;
		return $out;
	}
	return 0;
}

# Function : runsql
# Desc     : Execute SQLSTATEMENTS using client DBAMGR
# Input    : $Owner - owner of executables
#          : $LD_ENV - Env var string returned from function "check_environment"
#          : $DBAMGR - client utility
#          : $CONNECTSTR - client utility specific connectstring
#          : $TOLOG - (1) - the message has to be formatted with BEGLINE and ENDLINE
#          : $SQLSTATEMENTS - statements to execute
# Output   : None 
# Returns  : $out
#
sub runsql() {
	my($Owner, $LD_ENV,$DBAMGR,$CONNECTSTR,$TOLOG, $SQLSTATEMENTS) = @_;
	my $BEGLINE="\n+-----------------------------------------------------------" .
                                                        "---------+\n";
	my $ENDLINE="+===========================================================" .
                                                        "=========+";
	my $out = "";

	if (${SQLSTATEMENTS}) {
		$out = `$SU $Owner -c "$LD_ENV; echo $LIBRARY_PATH - \\\$$LIBRARY_PATH ; $DBAMGR" << !
${CONNECTSTR}
${SQLSTATEMENTS}
!`;
	}
	
	#Format messages if set.
	if (${TOLOG}) {
		$out = $BEGLINE . $out . $ENDLINE; 
	}

	return $out;

} # End of sub runrsql()

# Function : end_backup
# Desc     : Generate and execute SQLSTATEMENTS using client DBAMGR to end backup
# Input    : $Owner - owner of executables
#          : $ldenv - Env var string returned from function "check_environment"
#          : $dbamgr - client utility
#          : $connect- client utility specific connectstring
#          : $SID -  oracle instance
# Output   : None 
# Returns  : $out - output of alter database open
# Note     : It will be a good idea to add debug messages at every stage after exec runsql.
#
sub end_backup {
my($owner,$ldenv,$dbamgr,$connect,$SID) = @_;
my $SQLSTATEMENTS="";
#populate variables LD_ENV,DBAMGR,CONNECTSTR
                # Used in Oracle's "spool" statement below.
                #
                my $logfile="/tmp/online.${SID}.$$.log";

                unlink ${logfile}; # rm, if a file with that name is present

                ${SQLSTATEMENTS} = "set newpage none\n" .
                                   "set feedback off\n" .
                                   "set heading off\n"  .
                                   "spool ${logfile}\n" .
                                   "set echo on\n"      .
                "select file# from v\\\$backup where status = 'ACTIVE';\n" .
                                   "spool off;\n";

                &runsql($owner,$ldenv,$dbamgr,$connect,0, ${SQLSTATEMENTS});
		                # Open the $logfile and prepare "end backup" statements
                #
                my ($datafiles , $datafile , $datafile_cnt, $sqlline);
		$datafiles="";
                if (open(FD, "${logfile}")) {
                        while(<FD>) {
                                s/v\$/v\\\$/g;       # change v$ to v\$

                                s/\s+//g; # rm all white space, \n included
                                next if /^$/;

                                if (!/([\D]+)/) {    # Not a non-digit
                                        $datafile = $_;
                                        ${datafile_cnt}++;

                                        if (${datafiles} ne "") {
                                                ${datafiles} .= ", ${datafile}";
					} else {
                                                ${datafiles} .= "${datafile}";
                                        }
                                }
                                if (${datafile_cnt} > 49) {
		                        ${SQLSTATEMENTS}=&create_backup_sql(${datafiles});
               			 	&runsql($owner,$ldenv,$dbamgr,$connect,0, ${SQLSTATEMENTS});

                                        ${datafile_cnt} = 0;
                                        ${datafiles} = "";
                                }
                        }
                }
                close(FD);

                if (${datafile_cnt} != 0) {
                        ${SQLSTATEMENTS}=&create_backup_sql(${datafiles});
                	&runsql($owner,$ldenv,$dbamgr,$connect,0, ${SQLSTATEMENTS});
                }

                # Ok, now let's open the database
                #
                ${sqlline} = "alter database open";
                ${SQLSTATEMENTS} = "host echo ${sqlline}\n${sqlline};\n";
                my $output =&runsql($owner,$ldenv,$dbamgr,$connect,1, $SQLSTATEMENTS);

                unlink ${logfile}; # Comment this line for debugging
		return $output;


}

# Function : create_backup_sql
# Desc     : takes an input of datafiles and creates an SQLSTATEMENT to end backup.
# Input    : $datafiles - list of datafiles 
# Output   : None 
# Returns  : $SQLSTATEMENTS - sequence of statements to end backup
#
sub create_backup_sql {

	my($datafiles) = @_;

        my ${sqlline} = "alter database datafile ${datafiles} end backup";
        my ${SQLSTATEMENTS} = "host echo ${sqlline}\n${sqlline};\n";

	return ${SQLSTATEMENTS};
}

### Start of start database functions ##################################
# Function : startup_{options}
# Desc     : Generate SQLSTATEMENTS for various startup options
# Desc     : Use "runsql" to execute these statements 
# Input    : $Owner - owner of executables
#          : $ldenv - Env var string returned from function "check_environment"
#          : $dbamgr - client utility
#          : $connect- client utility specific connectstring
#          : $Pfile -  text initialization parameter file
# Output   : None 
# Returns  : $sqlout - output of startup statements
#

##
## SQL statement => startup
##
sub startup_database {

        my($owner,$ldenv, $dbamgr, $connect, $Pfile)=@_ ;
        my $SQLstmt="";
        my $sqlout="";

        if ( -e $Pfile and ( $Pfile ne "" or $Pfile ne " " )) {
                $SQLstmt="startup pfile=\'$Pfile\';\n";
        } else {
                $SQLstmt="startup ;\n";
        }
        $sqlout=&runsql($owner,$ldenv,$dbamgr,$connect,1,$SQLstmt);
        return $sqlout;
}

##
### SQL statement => startup force
###
sub startup_force_database {

        my($owner,$ldenv, $dbamgr, $connect, $Pfile)=@_ ;
        my $SQLstmt="";
        my $sqlout="";

	if ( -e $Pfile and ( $Pfile ne "" or $Pfile ne " " )) {
	        $SQLstmt="startup force pfile=\'$Pfile\';\n";
	} else {
	        $SQLstmt="startup force ;\n";
	}
        $sqlout=&runsql($owner,$ldenv,$dbamgr,$connect,1,$SQLstmt);
        return $sqlout;
}

##
## SQL statement => startup force mount; alter database recover;alter database open;
##
sub recover_database {

	my($owner,$ldenv, $dbamgr, $connect, $Pfile)=@_ ;

	my $SQLstmt="";

	if ( -e $Pfile and ( $Pfile ne "" or $Pfile ne " " )) {
		$SQLstmt="startup force pfile=\'$Pfile\' mount;\nalter database recover;\nalter database open;\n";
	} else {
		$SQLstmt="startup force mount;\nalter database recover;\nalter database open;\n"; 
	}

	my $sqlout=&runsql($owner,$ldenv,$dbamgr,$connect,1,$SQLstmt);
	return $sqlout;

}

##
## SQL statement => startup force restrict;
##
sub startup_restricted{

	my($owner,$ldenv, $dbamgr, $connect, $Pfile)=@_ ;

	my $SQLstmt="";

	if ( -e $Pfile and ( $Pfile ne "" or $Pfile ne " " )) {
		$SQLstmt="startup force pfile=\'$Pfile\' restrict;\n"; 
	} else {
		$SQLstmt="startup force restrict;\n"; 
	}

	my $sqlout=&runsql($owner,$ldenv,$dbamgr,$connect,1,$SQLstmt);
	return $sqlout;
}

##
## SQL statement => execute  start_custom_SID.sql
##
sub custom_startup {

	my($owner,$ldenv, $dbamgr, $connect, $SID)=@_ ;
	my $vcs_home=$AGENT_DIR;
	$vcs_home = "/opt/VRTSvcs" if (!$vcs_home);

	my $SQLstmt="";
	my $custom_script="$vcs_home/bin/Oracle/start_custom_$SID.sql";
	my $sqlout=0;

	if  ( -f $custom_script && -T $custom_script ) {

		$SQLstmt="@ $custom_script ;\n"; 
		$sqlout=&runsql($owner,$ldenv,$dbamgr,$connect,1,$SQLstmt);

	} else  {

		$sqlout=0;
	}

	return $sqlout;
}
### End of start database functions ##################################

# Function : srvctl_start_instance
# Desc     : Deternmine database name
# Desc     : start instance 
# Input    : $Owner - owner of executables
#          : $ldenv - Env var string returned from function "check_environment"
#          : $dbamgr - client utility
#          : $SID -  instance to be controlled
# Output   : None 
# Returns  : $out - output of "srvctl start" statements
#
sub srvctl_start_instance {

	my($owner,$ldenv, $dbamgr, $SID)=@_ ;
	my $BEGLINE="\n+-----------------------------------------------------------" .
                                                        "---------+\n";
	my $ENDLINE="+===========================================================" .
                                                        "=========+";
	my @instances;

	my $found = 0;
	my $db_sel = 0;
	my $out = 0;
	my @dbs = `$SU $owner -c "$ldenv; $dbamgr config database "` ;

	if ($? == 0 ) {
		for (@dbs) {
			$db_sel = $_;
			chomp($db_sel);
			@instances =`$SU $owner -c "$ldenv;$dbamgr config database -d $_"`;
			for (@instances) {
				if ( /\b$SID\b/) {
					$found = 1;	
					last ;
				}
				last if ($found) ;
			}
			last if ($found) ;
		}
	}

	if ($db_sel) {

		$out = `$SU $owner -c "$ldenv; $dbamgr start instance -d $db_sel -i $SID "` ;

	}	
	$out = $BEGLINE . $out . $BEGLINE if ($out) ;
	return $out;
}


# Function : srvctl_stop_instance
# Desc     : Deternmine database name
# Desc     : stop instance 
# Input    : $Owner - owner of executables
#          : $ldenv - Env var string returned from function "check_environment"
#          : $dbamgr - client utility
#          : $SID -  instance to be controlled
# Output   : None 
# Returns  : $out - output of "srvctl stop" statements
#
sub srvctl_stop_instance {

	my($owner,$ldenv, $dbamgr, $SID)=@_ ;
	my @instances;	
	my $found = 0;
	my $db_sel = 0;
	my $out = 0;
	my @dbs = `$SU $owner -c "$ldenv; $dbamgr config database "` ;

	if ($? == 0 ) {
		for (@dbs) {
			$db_sel = $_;
			chomp($db_sel);
			@instances =`$SU $owner -c "$ldenv;$dbamgr config database -d $_"`;
			for (@instances) {
				if ( /\b$SID\b/) {
					$found = 1;	
					last ;
				}
				last if ($found) ;
			}
			last if ($found) ;
		}
	}

	if ($db_sel) {

		$out = `$SU $owner -c "$ldenv; $dbamgr stop instance -d $db_sel -i $SID "` ;

	}	
	return $out;
}

### Start of shut database functions ##################################
# Function : shut_{options}
# Desc     : Generate SQLSTATEMENTS for various shutdown options
# Desc     : Use "runsql" to execute these statements 
# Input    : $Owner - owner of executables
#          : $ldenv - Env var string returned from function "check_environment"
#          : $dbamgr - client utility
#          : $connect- client utility specific connectstring
#          : $Pfile -   not really required,actually!
# Output   : None 
# Returns  : $sqlout - output of shutdown statements
#

##
## SQLSTAMEMENTS => shutdown immediate
##
sub shut_immediate_database {

	my($owner,$ldenv, $dbamgr, $connect, $Pfile)=@_ ;
	my $SQLstmt="";

	$SQLstmt="shutdown immediate ;\n"; 
	my $sqlout=&runsql($owner,$ldenv,$dbamgr,$connect,1,$SQLstmt);
	return $sqlout;
}

##
## SQLSTAMEMENTS => shutdown transactional
##
sub shut_transactional_database {

	my($owner,$ldenv, $dbamgr, $connect, $Pfile)=@_ ;
	my $SQLstmt="";

	$SQLstmt="shutdown transactional;\n"; 
	my $sqlout=&runsql($owner,$ldenv,$dbamgr,$connect,1,$SQLstmt);
	return $sqlout;
}


##
## SQLSTAMEMENTS => shutdown abort
##
sub shut_abort_database {

	my($owner,$ldenv, $dbamgr, $connect, $Pfile)=@_ ;

	my $SQLstmt="";

	$SQLstmt="shutdown abort;\n"; 
	my $sqlout=&runsql($owner,$ldenv,$dbamgr,$connect,1,$SQLstmt);
	return $sqlout;
}

##
## SQLSTAMEMENTS => execute shut_custom_SID.sql
##
sub custom_shutdown {

	my($owner,$ldenv, $dbamgr, $connect, $SID)=@_ ;
	my $vcs_home=$AGENT_DIR;
	$vcs_home = "/opt/VRTSvcs" if (!$vcs_home);
        my $SQLstmt="";
        my $custom_script="$vcs_home/bin/Oracle/shut_custom_$SID.sql";
        my $sqlout=0;

        if  ( -f $custom_script && -T $custom_script ) {

                $SQLstmt="@ $custom_script ;\n";
                $sqlout=&runsql($owner,$ldenv,$dbamgr,$connect,1,$SQLstmt);

        } else  {

                $sqlout=0;
        }

	return $sqlout;
}
### End of shut database functions ##################################

# Function : parse_error_output
# Desc     : Format error string to pass to function within C code which takes appropriate action
# Input    : $SQLError - error output
# Output   : None 
# Returns  : $error_string - space separetd list of oracle error codes
#
sub parse_error_output {

	my($SQLError)=@_ ;
	my($error_string) = "";

        my @lines = split(/\n+/, $SQLError);
       	my ($i,$ora_error,$rem,$errno); 
        for ($i=0; $i < @lines; $i++) {
                $_ = $lines[$i];
                if (/.*ORA-/) {
			($ora_error,$rem) =split/:/ ;
			($rem,$errno) = split/-/,$ora_error ;
			$error_string = $errno . " " . $error_string ;
		}
        }
	
	return $error_string;
}

# Function : get_pid_list
# Desc     : get list of pids specific to an instance
# Input    : $SID
# Output   : None
# Returns  : @all_pids
#
sub get_pid_list {

        my ($SID) = @_;
        my ($processname);
        my ($pid_one);
        my (@all_pids);
        unless (open(PS, "$PS | $GREP \"ora_\"|")) {
                push(@all_pids,"-1");
                return @all_pids;
        }
        while (<PS>){
                ($pid_one, $processname) = split " ",$_,2;

                if ($processname =~ /^ora_.*_$SID\b/) {
                        push(@all_pids,$pid_one);
                }
        }
        close PS;
        return @all_pids;
}

# Function : get_asm_pid_list
# Desc     : get list of pids specific to an instance
# Input    : $SID
# Output   : None
# Returns  : @all_pids
#
sub get_asm_pid_list {

        my ($SID) = @_;
        my $mSID = "\\$SID";
        my $processname="";
        my ($pid_one);
        my (@all_pids);
        unless (open(PS, "$PS | $GREP \"asm_\"|")) {
                push(@all_pids,"-1");
                return @all_pids;
        }
        while (<PS>){
                ($pid_one, $processname) = split " ",$_,2;
            
		if ($processname =~ /^asm_.*_$mSID\b/) {    
                        push(@all_pids, $pid_one);
                }
        }
        close PS;
        return @all_pids;
}

# Function : check_oracle_daemon
# Desc     : check if the passed processname is running.
# Input    : Home, processname
# Output   : None
# Returns  : 0 (open fail or not found) else 1.
sub check_oracle_daemon {

        my ($Home, $processname) = @_;
        my ($returnval);
        my (@result);

	open(PS, "$PS | $GREP \"$processname\" | $GREP -v $GREP |");
	@result = <PS>;

	if ($#result == -1){
		$returnval = 0;
	}
	else {
		$returnval = 1;
	}		
	close PS;
	return $returnval;
}

sub find_version {
	my ($Home, $version, $release) = @_;
	my ($out, @out1);
	my ($lineval, $string, $str);
	
	$ENV{'ORACLE_HOME'} = $Home;
	$out = `$Home/bin/sqlplus -V`;
	@out1 = split(/\n/,$out);
	foreach $lineval (@out1) {
		chomp $lineval;
		if($lineval=~/Release/) {
			$string = $lineval;
			($str = $string) =~ s/.*Release\s+(\d+)\.(\d+).*/$1/;
			$release = $2;
			last;
		}
	}
	$_[1] = $str;
	$_[2] = $release;
	return 1;
}
	
sub start_cssd_process {
	my ($Home) = @_;
	my $out = "";
	my $ret;
	$out = `$Home/bin/crsctl start resource ora.cssd`;
	if ( $? == 0) {
		$ret = 0;
	} else {
		$ret = 1;
	}
	return $ret;
}

1;
